<?php

namespace kernel\app_modules\tag\service;

use Illuminate\Database\Eloquent\Model;
use kernel\app_modules\tag\models\Tag;
use kernel\app_modules\tag\models\TagEntity;
use kernel\FormModel;
use kernel\helpers\Debug;
use kernel\helpers\Slug;
use kernel\services\ModuleService;

class TagService
{
    public function create(FormModel $form_model): false|Tag
    {
        $model = new Tag();
        $model->label = $form_model->getItem('label');
        $model->entity = $form_model->getItem('entity');
        $model->status = $form_model->getItem('status');
        $model->slug = Slug::createSlug($form_model->getItem('label'), Tag::class);
        if ($model->save()){
            return $model;
        }

        return false;
    }

    public function update(FormModel $form_model, Tag $tag): false|Tag
    {
        if ($tag->label !== $form_model->getItem('label')) {
            $tag->slug = Slug::createSlug($form_model->getItem('label'), Tag::class);
        }
        $tag->label = $form_model->getItem('label');
        $tag->entity = $form_model->getItem('entity');
        $tag->status = $form_model->getItem('status');

        if ($tag->save()){
            return $tag;
        }

        return false;
    }


    public static function getEntityByTagId(int $tagId, $model)
    {
        $tagEntity = TagEntity::where("tag_id", $tagId)->get();
        if ($tagEntity){
            $entityIdArr = [];
            foreach ($tagEntity as $item){
                $entityIdArr[] = $item['entity_id'];
            }
        }
        $queryBuilder = $model::query();

        return $queryBuilder->whereIn("id", $entityIdArr)->get();
    }

    public static function getEntityByTagSlug(string $tagSlug, $model)
    {
        $tag = Tag::where("slug", $tagSlug)->first();
        if ($tag){
            $tagEntity = TagEntity::where("tag_id", $tag->id)->get();
            if ($tagEntity){
                $entityIdArr = [];
                foreach ($tagEntity as $item){
                    $entityIdArr[] = $item['entity_id'];
                }
            }
            $queryBuilder = $model::query();

            return $queryBuilder->whereIn("id", $entityIdArr)->get();
        }

        return [];
    }

}